## TODO: remove paths to personal scratchs 
source("/esarchive/scratch/vagudets/repos/csoperational/R/get_regrid_params.R")

lapse_rate_correction <- function(recipe) {
  
  ## Load exp and obs data
  source("modules/Loading/R/load_seasonal.R")
  data <- load_seasonal(recipe)
  
  ref.name <- recipe$Analysis$Datasets$Reference$name
  exp.name <- recipe$Analysis$Datasets$System$name
  
  archive <- read_yaml("conf/archive.yml")$esarchive
  
  # Define regrid parameters:
  regrid_params <- get_regrid_params(recipe, archive)
  
  ## Load exp orography
  orography_exp <- Start(dat = archive$System[[exp.name]]$orography, 
                         var = 'orography',  
                         lon = 'all',
                         lat = 'all',
                         transform = regrid_params$fcst.transform,
                         transform_params = list(grid = regrid_params$fcst.gridtype,
                                                 method = regrid_params$fcst.gridmethod),
                         transform_vars = c('lat','lon'),
                         return_vars = list(lat = NULL, lon = NULL), 
                         synonims = list(lon = c('lon','longitude'),
                                         lat = c('lat','latitude')),
                         lat_reorder = Sort(decreasing = TRUE),
                         lon_reorder = CircularSort(0,359.9),
                         num_procs = 1, retrieve = TRUE)
  
  ## Set negative values to zero
  orography_exp <- pmax(orography_exp, 0)
  
  ## load obs orography
  orography_obs <- Start(dat = archive$Reference[[ref.name]]$orography, 
                         var = 'orography',  
                         lon = 'all',
                         lat = 'all',
                         transform = regrid_params$obs.transform,
                         transform_params = list(grid = regrid_params$obs.gridtype,
                                                 method = regrid_params$obs.gridmethod),
                         transform_vars = c('lat','lon'),
                         return_vars = list(lat = NULL, lon = NULL), 
                         synonims = list(lon = c('lon','longitude'),
                                         lat = c('lat','latitude')),
                         lat_reorder = Sort(decreasing = TRUE),
                         lon_reorder = CircularSort(0,359.9),
                         num_procs = 1, retrieve = TRUE)
  
  ## Set negative values to zero
  orography_obs <- pmax(orography_obs, 0)
  
  ## Calculate difference between orographies
  oro_diff <- orography_exp - orography_obs
  
  ## Apply lapse rate factor to correct temperature = -0.0065 K/m (-6.5 K/km)
  
  oro_obs_corr <- oro_diff * -0.0065
  oro_obs_corr <- Reorder(data = drop(oro_obs_corr), order = c('lat', 'lon'))
  
  # corrected_obs <- data$obs$data
  
  ## Apply correction to obs temperature data
  for(ens in 1:dim(data$obs$data)['ensemble']){
    for(time in 1:dim(data$obs$data)['time']){
      for(syear in 1:dim(data$obs$data)['syear']){
        data$obs$data[,,,,syear,time,,,ens] <- data$obs$data[,,,,syear,time,,,ens] + oro_obs_corr
        
      }
    }  
  }
  
  # mean_obs_orig <- Apply(data$obs$data, target_dims = 'syear', fun = 'mean')$output1
  # 
  # mean_obs_corr <- Apply(corrected_obs, target_dims = 'syear', fun = 'mean')$output1
  # 
  # diff_temp <-  mean_obs_orig - mean_obs_corr
  
  
}

